package es.kauron.jstudy.controller;

import es.kauron.jstudy.model.AppPrefs;
import es.kauron.jstudy.model.TestItem;
import es.kauron.jstudy.util.Clock;
import javafx.beans.binding.Bindings;
import javafx.beans.property.*;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.*;
import javafx.scene.layout.Pane;

import java.net.URL;
import java.util.List;
import java.util.ResourceBundle;

public class TestController implements Initializable {
    private static final TestItem ending = new TestItem("That's it!", "");

    @FXML
    private Label question, prevQuestion, prevAnswer, correctAnswer, progressLabel, correctLabel;
    @FXML
    private TextField answer;
    @FXML
    private ProgressBar progress;
    @FXML
    private Pane feedback, buttonBox;
    @FXML
    private Button skipButton;
    @FXML
    private CheckBox pauseCheckBox;

    private final SimpleBooleanProperty correctingError = new SimpleBooleanProperty(false);
    private List<TestItem> list;
    private final IntegerProperty errors = new SimpleIntegerProperty(0);
    private final ObjectProperty<TestItem> item = new SimpleObjectProperty<>();
    private final IntegerProperty done = new SimpleIntegerProperty(0);
    // Time accounting
    private final Clock clock = new Clock();

    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        skipButton.disableProperty().bind(correctingError);
        feedback.visibleProperty().bind(AppPrefs.showFeedback);
        answer.disableProperty().bind(pauseCheckBox.selectedProperty());
        buttonBox.disableProperty().bind(pauseCheckBox.selectedProperty());
        pauseCheckBox.selectedProperty().addListener((obj, o, n) -> {
            if (n) clock.pause();
            else   clock.play();
        });
    }

    void setList(List<TestItem> list) {
        this.list = list;
        int total = list.size();
        progressLabel.textProperty().bind(Bindings.format(
                String.format("%%.2f%%%%     %%d / %d     %%d mistakes     %%s", total),
                done.multiply(100.0).divide(total), done,
                errors, clock.timeProperty()));
        progress.progressProperty().bind(done.divide((double) total));
        item.addListener((obj, o, n) -> {
            if (o != null) {
                prevQuestion.setText(o.getQuestion());
                correctAnswer.setText(o.getAnswer());
            }
            question.setText(n.getQuestion());
        });
        clock.play();
        onSkipAction(null);
    }

    @FXML
    private void onNextAction(ActionEvent event) {
        prevAnswer.setText(answer.getText());

        boolean right = item.get().checkAnswer(answer.getText());
        correctAnswer.setVisible(!right);
        correctLabel.setVisible(!right);

        if (!right) {
            errors.set(errors.get() + 1);
            prevAnswer.setStyle("-fx-text-fill: #C40000;");
        } else {
            prevAnswer.setStyle("-fx-text-fill: #00C400;");
        }

        // Remove the question from the pool if the question was correctly answered or there is no repetition
        if (right || !AppPrefs.repeatWrong.get()) {
            if (!correctingError.get())
                done.set(done.get() + 1);
            correctingError.set(!right && AppPrefs.repeatImmediately.get());
            list.remove(item.get());
            if (list.size() == 0) {
                onEndAction(null);
                return;
            }
            chooseQuestion();
        } else if (AppPrefs.repeatImmediately.get()) {
            correctingError.set(true);
            item.set(new TestItem(item.get()));
            list.add(item.get());
        } else {
            chooseQuestion();
        }
        answer.setText("");
        answer.requestFocus();
    }

    private void chooseQuestion() {
        answer.setText("");
        TestItem next;
        do
            next = list.get((int) (Math.random() * list.size()));
        while (list.size() > 1 && item.get() == next);
        item.set(next);
    }

    @FXML
    private void onSkipAction(ActionEvent event) {
        chooseQuestion();
        answer.setText("");
        answer.requestFocus();
    }

    @FXML
    private void onEndAction(ActionEvent event) {
        answer.setText("");
        item.set(ending);
        pauseCheckBox.setSelected(true);
        pauseCheckBox.setDisable(true);
    }
}
